-- ---------------------
-- NeedToKnow
-- by Nephthys of Hyjal
-- lieandswell@yahoo.com
-- ---------------------


-- -------------
-- ADDON GLOBALS
-- -------------

NeedToKnow = {};
-- NEEDTOKNOW = {} is defined in the localization file, which must be loaded before this file

NEEDTOKNOW.VERSION = "2.1";
NEEDTOKNOW.MAXGROUPS = 4;
NEEDTOKNOW.MAXBARS = 6;
NEEDTOKNOW.UPDATE_INTERVAL = 0.05;

NEEDTOKNOW.BAR_DEFAULTS = {
	Enabled			= true,
	AuraName		= "",
	--	AuraName		= {""},
	--	Priority		= "list" or "maxtime" or "mintime",
	Unit			= "player",
	BuffOrDebuff	= "HELPFUL",
	OnlyMine		= true,
	BarColor		= { r=0.6, g=0.6, b=0.6, a=1.0 },
};
NEEDTOKNOW.GROUP_DEFAULTS = {
	Enabled		= false,
	NumberBars	= 3,
	Scale		= 1.0,
	Width		= 270,
	Bars		= {},
	Position	= { "TOPLEFT", "TOPLEFT", 100, -100 },
};
NEEDTOKNOW.DEFAULTS = {
	Version		= NEEDTOKNOW.VERSION,
	Locked		= false,
	BarTexture	= "BantoBar",
	BkgdColor	= { 0, 0, 0, 0.8 },
	BarSpacing	= 3,
	BarPadding	= 3,
	Groups		= {},
};
for barID = 1, NEEDTOKNOW.MAXBARS do
	NEEDTOKNOW.GROUP_DEFAULTS["Bars"][barID] = NEEDTOKNOW.BAR_DEFAULTS;
end;
for groupID = 1, NEEDTOKNOW.MAXGROUPS do
	NEEDTOKNOW.DEFAULTS["Groups"][groupID] = NEEDTOKNOW.GROUP_DEFAULTS;
end;

function NeedToKnow.Test(stuff)
	if ( stuff ) then
		DEFAULT_CHAT_FRAME:AddMessage("NeedToKnow test: "..stuff);
	else
		DEFAULT_CHAT_FRAME:AddMessage("NeedToKnow test: "..self:GetName());
	end
end



-- ---------------
-- EXECUTIVE FRAME
-- ---------------

function NeedToKnow.ExecutiveFrame_OnEvent(self, event)
	if ( event == 'VARIABLES_LOADED' ) then
		if ( not NeedToKnow_Settings ) then
			NeedToKnow_Settings = CopyTable(NEEDTOKNOW.DEFAULTS);
			NeedToKnow_Settings["Groups"][1]["Enabled"] = true;
		elseif ( NeedToKnow_Settings["Version"] < NEEDTOKNOW.VERSION ) then
			NeedToKnow.SafeUpgrade();
		end
		SlashCmdList["NEEDTOKNOW"] = NeedToKnow.SlashCommand;
		SLASH_NEEDTOKNOW1 = "/needtoknow";
		SLASH_NEEDTOKNOW2 = "/ntk";
	elseif ( event == "PLAYER_ENTERING_WORLD" ) or ( event == "PLAYER_LOGIN" ) then
		-- save group positions if upgrading from version that used layout-local.txt
		if ( NeedToKnow_Settings["Version"] < "2.1" ) then	
			for groupID = 1, NEEDTOKNOW.MAXGROUPS do
				NeedToKnow.SavePosition(getglobal("NeedToKnow_Group"..groupID), groupID)
			end
		end
		NeedToKnow.Update();
	end
end

function NeedToKnow.SafeUpgrade()
	if ( NeedToKnow_Settings["Version"] < "2.0" ) then			-- total settings clear if v1.x
		NeedToKnow_Settings = CopyTable(NEEDTOKNOW.DEFAULTS);
		NeedToKnow_Settings["Groups"][1]["Enabled"] = true;
	else
		NeedToKnow_Settings = NeedToKnow.AddNewSettings(NeedToKnow_Settings, NEEDTOKNOW.DEFAULTS);
		NeedToKnow_Settings["Version"] = NEEDTOKNOW.VERSION;
	end
end

function NeedToKnow.AddNewSettings(settings, defaults)
	for k, v in pairs(defaults) do
		if ( not settings[k] ) then
			if ( type(v) == "table" ) then
				settings[k] = {};
				settings[k] = NeedToKnow.AddNewSettings(settings[k], defaults[k]);
			else
				settings[k] = v;
			end

		--[[
		elseif ( type(v) == "table" ) and ( type(settings[k]) ~= "table" ) then
			local oldSetting = settings[k];
			settings[k] = {};
			settings[k][1] = oldSetting;
			--	settings[k] = {settings[k]};
		--]]

		elseif ( type(v) == "table" ) then
			settings[k] = NeedToKnow.AddNewSettings(settings[k], defaults[k]);
		end
	end
	return settings;
end

function NeedToKnow.Update()
	for groupID = 1, NEEDTOKNOW.MAXGROUPS do
		NeedToKnow.Group_Update(groupID);
	end
end

do
	local executiveFrame = CreateFrame("Frame", "NeedToKnow_ExecutiveFrame");
	executiveFrame:SetScript("OnEvent", NeedToKnow.ExecutiveFrame_OnEvent);
	executiveFrame:RegisterEvent("VARIABLES_LOADED");
	executiveFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
	executiveFrame:RegisterEvent("PLAYER_LOGIN");
end



-- ------
-- GROUPS
-- ------

function NeedToKnow.Group_Update(groupID)
	local groupName = "NeedToKnow_Group"..groupID;
	local group = getglobal(groupName);
	local groupSettings = NeedToKnow_Settings["Groups"][groupID];

	for barID = 1, groupSettings.NumberBars do
		local barName = groupName.."Bar"..barID;
		local bar = getglobal(barName) or CreateFrame("StatusBar", barName, group, "NeedToKnow_BarTemplate");
		bar:SetID(barID);

		if ( barID > 1 ) then
			bar:SetPoint("TOP", getglobal(groupName.."Bar"..(barID-1)), "BOTTOM", 0, -NeedToKnow_Settings.BarSpacing);
		else
			bar:SetPoint("TOPLEFT", group, "TOPLEFT");
		end

		NeedToKnow.Bar_Update(groupID, barID);

		if ( not groupSettings.Enabled ) then
			NeedToKnow.ClearScripts(bar);
		end
	end
	for barID = groupSettings.NumberBars+1, NEEDTOKNOW.MAXBARS do
		local bar = getglobal(groupName.."Bar"..barID);
		if ( bar ) then
			bar:Hide();
			NeedToKnow.ClearScripts(bar);
		end
	end

	local resizeButton = getglobal(groupName.."ResizeButton");
	resizeButton:SetPoint("BOTTOMRIGHT", groupName.."Bar"..groupSettings.NumberBars, "BOTTOMRIGHT", 8, -8);
	if ( NeedToKnow_Settings["Locked"] ) then
		resizeButton:Hide();
	else
		resizeButton:Show();
	end

	group:ClearAllPoints();
	local point, relativePoint, xOfs, yOfs = unpack(groupSettings.Position);
	group:SetPoint(point, UIParent, relativePoint, xOfs, yOfs);
	group:SetScale(groupSettings.Scale);
	if ( groupSettings.Enabled ) then
		group:Show();
	else
		group:Hide();
	end
end



-- ----
-- BARS
-- ----

function NeedToKnow.Bar_Update(groupID, barID)
	local barName = "NeedToKnow_Group"..groupID.."Bar"..barID;
	local bar = getglobal(barName);
	local background = getglobal(barName.."Background");
	bar.spark = getglobal(barName.."Spark");
	bar.text = getglobal(barName.."Text");
	bar.time = getglobal(barName.."Time");

	local groupSettings = NeedToKnow_Settings["Groups"][groupID];
	local barSettings = groupSettings["Bars"][barID];
	bar.auraName = barSettings.AuraName;
	bar.buffOrDebuff = barSettings.BuffOrDebuff;
	bar.unit = barSettings.Unit;
	bar.onlyMine = barSettings.OnlyMine;
	bar.updateTimer = NEEDTOKNOW.UPDATE_INTERVAL;
	bar.updateInterval = NEEDTOKNOW.UPDATE_INTERVAL;

	bar:SetMinMaxValues(0,1);
	bar:SetValue(1);
	bar:SetStatusBarTexture("Interface\\AddOns\\NeedToKnow\\Textures\\"..NeedToKnow_Settings["BarTexture"]);
	bar:SetStatusBarColor(barSettings.BarColor.r, barSettings.BarColor.g, barSettings.BarColor.b, barSettings.BarColor.a);

	bar:SetWidth(groupSettings.Width);
	bar.text:SetWidth(groupSettings.Width-60);
	background:SetWidth(groupSettings.Width + 2*NeedToKnow_Settings["BarPadding"]);
	background:SetHeight(bar:GetHeight() + 2*NeedToKnow_Settings["BarPadding"]);
	background:SetVertexColor(unpack(NeedToKnow_Settings["BkgdColor"]));

	bar.text:SetText(bar.auraName);
	if ( NeedToKnow_Settings["Locked"] ) then
		bar:EnableMouse(0);
		bar:Hide();
		bar.spark:Show();
		bar.time:Show();
	else
		NeedToKnow.ClearScripts(bar);
		bar:EnableMouse(1);
		bar:Show();
		bar.spark:Hide();
		bar.time:Hide();
	end

	if ( barSettings.Enabled ) then
		bar:SetAlpha(1);
	else
		bar:SetAlpha(0.4);
	end

	if ( NeedToKnow_Settings["Locked"] and barSettings.Enabled ) then
		NeedToKnow.SetScripts(bar);
		NeedToKnow.Bar_AuraCheck(bar);		-- a check for when the addon gets locked
	else
		NeedToKnow.ClearScripts(bar);
	end
end

function NeedToKnow.SetScripts(bar)
	bar:SetScript("OnEvent", NeedToKnow.Bar_OnEvent);
	bar:SetScript("OnUpdate", NeedToKnow.Bar_OnUpdate);
	bar:RegisterEvent("PLAYER_TARGET_CHANGED");
	bar:RegisterEvent("PLAYER_FOCUS_CHANGED");
	bar:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	bar:RegisterEvent("UNIT_AURA");
end

function NeedToKnow.ClearScripts(bar)
	bar:SetScript("OnEvent", nil);
	bar:SetScript("OnUpdate", nil);
	bar:UnregisterEvent("PLAYER_TARGET_CHANGED");
	bar:UnregisterEvent("PLAYER_FOCUS_CHANGED");
	bar:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	bar:UnregisterEvent("UNIT_AURA");
end

function NeedToKnow.Bar_OnEvent(self, event, ...)
--[[
	if ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
--		local timestamp, combatEvent, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags;
--		local spellID, spellName, spellSchool, auraType;
		local combatEvent = select(2, ...);
		if ( combatEvent == "SPELL_AURA_APPLIED" ) 
			or ( combatEvent == "SPELL_AURA_REMOVED" ) 
			or ( combatEvent == "SPELL_AURA_APPLIED_DOSE" ) 
			or ( combatEvent == "SPELL_AURA_REMOVED_DOSE" ) 
			or ( combatEvent == "SPELL_AURA_REFRESH" ) 
			or ( combatEvent == "SPELL_AURA_BROKEN" ) 
			or ( combatEvent == "SPELL_AURA_BROKEN_SPELL" ) 
		then
			if ( select(10, ...) == self.auraName ) then	-- could add check for unitGUID here, i guess.  meh.  
				NeedToKnow.Bar_AuraCheck(self);
			end
		elseif ( combatEvent == "UNIT_DIED" ) then
			if ( select(6, ...) == UnitGUID(self.unit) ) then
				NeedToKnow.Bar_AuraCheck(self);
			end
		end
--]]
	if ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) and ( select(2, ...) == "UNIT_DIED" ) then
		if ( select(6, ...) == UnitGUID(self.unit) ) then
			NeedToKnow.Bar_AuraCheck(self);
		end
	elseif ( event == "UNIT_AURA" ) and ( select(1, ...) == self.unit ) then
		NeedToKnow.Bar_AuraCheck(self);
	elseif ( event == "PLAYER_TARGET_CHANGED" ) or ( event == "PLAYER_FOCUS_CHANGED" ) then
		NeedToKnow.Bar_AuraCheck(self);
	end
end

function NeedToKnow.Bar_AuraCheck(bar)
	if ( UnitExists(bar.unit) ) then
		local buffName, _, _, count, _, duration, expirationTime, caster = UnitAura(bar.unit, bar.auraName, nil, bar.buffOrDebuff);
		if ( not duration ) then
			bar:Hide();
			return;
		elseif ( ( caster == "player" ) or ( caster == "pet" ) or ( caster == "vehicle" ) or not bar.onlyMine ) then
			bar.duration = duration;
			bar.timeLeft = expirationTime - GetTime();
			bar:SetMinMaxValues(0, duration);
			bar:SetValue(bar.timeLeft);
			bar.spark:SetPoint("CENTER", bar, "LEFT", bar:GetWidth()*bar.timeLeft/bar.duration, 0);
			if ( count > 1 ) then
				bar.text:SetText(bar.auraName.."  ["..count.."]");
			else
				bar.text:SetText(bar.auraName);
			end
			bar:Show();
			return;
		end
		bar:Hide();		-- move along.  these aren't the buffs you're looking for.  
	else
		bar:Hide();
	end
end

function NeedToKnow.Bar_OnUpdate(self, elapsed)
	self.updateTimer = self.updateTimer - elapsed;
	self.timeLeft = self.timeLeft - elapsed;
	if ( self.updateTimer <= 0 ) then 
		self.updateTimer = self.updateInterval;
		self.timeLeft = max(0, self.timeLeft);
		self:SetValue(self.timeLeft);
		self.time:SetFormattedText(SecondsToTimeAbbrev(self.timeLeft));
		self.spark:SetPoint("CENTER", self, "LEFT", self:GetWidth()*self.timeLeft/self.duration, 0);
	end
end

